@@ -1,6 +1,20 @@
 """Defines the base dialect class."""
 
-from sqlfluff.core.parser import KeywordSegment, SegmentGenerator
+from typing import Union, Type
+
+from sqlfluff.core.parser import (
+    KeywordSegment,
+    SegmentGenerator,
+    BaseSegment,
+    StringParser,
+)
+from sqlfluff.core.parser.grammar.base import BaseGrammar
+
+DialectElementType = Union[
+    Type[BaseSegment], BaseGrammar, StringParser, SegmentGenerator
+]
+# NOTE: Post expansion, no generators remain
+ExpandedDialectElementType = Union[Type[BaseSegment], StringParser, BaseGrammar]
 
 
 class Dialect:
@@ -8,7 +22,7 @@ class Dialect:
 
     Args:
         name (:obj:`str`): The name of the dialect, used for lookup.
-        lexer_struct (iterable of :obj:`tuple`): A structure defining
+        lexer_matchers (iterable of :obj:`StringLexer`): A structure defining
             the lexing config for this dialect.
 
     """
@@ -16,7 +30,7 @@ class Dialect:
     def __init__(
         self,
         name,
-        lexer_struct=None,
+        lexer_matchers=None,
         library=None,
         sets=None,
         inherits_from=None,
@@ -24,43 +38,56 @@ class Dialect:
     ):
         self._library = library or {}
         self.name = name
-        self.lexer_struct = lexer_struct
+        self.lexer_matchers = lexer_matchers
         self.expanded = False
         self._sets = sets or {}
         self.inherits_from = inherits_from
         self.root_segment_name = root_segment_name
 
     def __repr__(self):
-        return "<Dialect: {0}>".format(self.name)
+        return f"<Dialect: {self.name}>"
 
-    def expand(self):
+    def expand(self) -> "Dialect":
         """Expand any callable references to concrete ones.
 
         This must be called before using the dialect. But
         allows more flexible definitions to happen at runtime.
 
+        NOTE: This method returns a copy of the current dialect
+        so that we don't pollute the original dialect and get
+        dependency issues.
+
+
+        Returns:
+            :obj:`Dialect`: a copy of the given dialect but
+                with expanded references.
         """
         # Are we already expanded?
         if self.expanded:
-            return
+            raise ValueError("Attempted to re-expand an already expanded dialect.")
+
+        expanded_copy = self.copy_as(name=self.name)
         # Expand any callable elements of the dialect.
-        for key in self._library:
-            if isinstance(self._library[key], SegmentGenerator):
+        for key in expanded_copy._library:
+            if isinstance(expanded_copy._library[key], SegmentGenerator):
                 # If the element is callable, call it passing the current
                 # dialect and store the result in its place.
                 # Use the .replace() method for its error handling.
-                self.replace(**{key: self._library[key].expand(self)})
+                expanded_copy.replace(
+                    **{key: expanded_copy._library[key].expand(expanded_copy)}
+                )
         # Expand any keyword sets.
         for keyword_set in [
             "unreserved_keywords",
             "reserved_keywords",
         ]:  # e.g. reserved_keywords, (JOIN, ...)
             # Make sure the values are available as KeywordSegments
-            for kw in self.sets(keyword_set):
+            for kw in expanded_copy.sets(keyword_set):
                 n = kw.capitalize() + "KeywordSegment"
-                if n not in self._library:
-                    self._library[n] = KeywordSegment.make(kw.lower())
-        self.expanded = True
+                if n not in expanded_copy._library:
+                    expanded_copy._library[n] = StringParser(kw.lower(), KeywordSegment)
+        expanded_copy.expanded = True
+        return expanded_copy
 
     def sets(self, label):
         """Allows access to sets belonging to this dialect.
@@ -80,6 +107,12 @@ class Dialect:
         This is the primary method for inheritance, after which, the
         `replace` method can be used to override particular rules.
         """
+        # Are we already expanded?
+        if self.expanded:
+            # If we copy an already expanded dialect then any SegmentGenerators
+            # won't respond. This is most likely a mistake.
+            raise ValueError("Attempted to copy an already expanded dialect.")
+
         # Copy sets if they are passed, so they can be mutated independently
         new_sets = {}
         for label in self._sets:
@@ -88,7 +121,7 @@ class Dialect:
         return self.__class__(
             name=name,
             library=self._library.copy(),
-            lexer_struct=self.lexer_struct.copy(),
+            lexer_matchers=self.lexer_matchers.copy(),
             sets=new_sets,
             inherits_from=self.name,
             root_segment_name=self.root_segment_name,
@@ -109,14 +142,10 @@ class Dialect:
             n = cls.__name__
             if replace:
                 if n not in self._library:
-                    raise ValueError(
-                        "{0!r} is not already registered in {1!r}".format(n, self)
-                    )
+                    raise ValueError(f"{n!r} is not already registered in {self!r}")
             else:
                 if n in self._library:
-                    raise ValueError(
-                        "{0!r} is already registered in {1!r}".format(n, self)
-                    )
+                    raise ValueError(f"{n!r} is already registered in {self!r}")
             self._library[n] = cls
             # Pass it back after registering it
             return cls
@@ -124,39 +153,66 @@ class Dialect:
         # return the wrapping function
         return segment_wrap
 
-    def add(self, **kwargs):
+    def add(self, **kwargs: DialectElementType):
         """Add a segment to the dialect directly.
 
         This is the alternative to the decorator route, most useful for segments
         defined using `make`. Segments are passed in as kwargs.
 
         e.g.
-        dialect.add(SomeSegment=KeywordSegment.make(blah, blah, blah))
+        dialect.add(SomeSegment=StringParser("blah", KeywordSegment))
 
         Note that multiple segments can be added in the same call as this method
         will iterate through the kwargs
         """
         for n in kwargs:
             if n in self._library:
-                raise ValueError("{0!r} is already registered in {1!r}".format(n, self))
+                raise ValueError(f"{n!r} is already registered in {self!r}")
             self._library[n] = kwargs[n]
 
-    def replace(self, **kwargs):
+    def replace(self, **kwargs: DialectElementType):
         """Override a segment on the dialect directly.
 
         Usage is very similar to add, but elements specified must already exist.
         """
         for n in kwargs:
             if n not in self._library:
-                raise ValueError(
-                    "{0!r} is not already registered in {1!r}".format(n, self)
-                )
+                raise ValueError(f"{n!r} is not already registered in {self!r}")
             self._library[n] = kwargs[n]
 
-    def ref(self, name):
+    def get_grammar(self, name: str) -> BaseGrammar:
+        """Allow access to grammars pre-expansion.
+
+        This is typically for dialect inheritance. This method
+        also validates that the result is a grammar.
+        """
+        if name not in self._library:
+            raise ValueError(f"Element {name} not found in dialect.")
+        if not isinstance(self._library[name], BaseGrammar):
+            raise TypeError(
+                f"Attempted to fetch non grammar [{name}] with get_grammar."
+            )
+        return self._library[name]
+
+    def get_segment(self, name: str) -> Type["BaseSegment"]:
+        """Allow access to segments pre-expansion.
+
+        This is typically for dialect inheritance. This method
+        also validates that the result is a segment.
+        """
+        if name not in self._library:
+            raise ValueError(f"Element {name} not found in dialect.")
+        if not issubclass(self._library[name], BaseSegment):
+            raise TypeError(
+                f"Attempted to fetch non segment [{name}] with get_segment."
+            )
+        return self._library[name]
+
+    def ref(self, name: str) -> ExpandedDialectElementType:
         """Return an object which acts as a late binding reference to the element named.
 
-        NB: This requires the dialect to be expanded.
+        NB: This requires the dialect to be expanded, and only returns Matchables
+        as a result.
 
         """
         if not self.expanded:
@@ -168,57 +224,55 @@ class Dialect:
                 return res
             else:
                 raise ValueError(
-                    "Unexpected Null response while fetching {0!r} from {1}".format(
+                    "Unexpected Null response while fetching {!r} from {}".format(
                         name, self.name
                     )
                 )
         else:
             raise RuntimeError(
-                "Grammar refers to {0!r} which was not found in the {1} dialect".format(
+                "Grammar refers to {!r} which was not found in the {} dialect".format(
                     name, self.name
                 )
             )
 
-    def set_lexer_struct(self, lexer_struct):
+    def set_lexer_matchers(self, lexer_matchers):
         """Set the lexer struct for the dialect.
 
         This is what is used for base dialects. For derived dialects
         (which don't exist yet) the assumption is that we'll introduce
         some kind of *patch* function which could be used to mutate
-        an existing `lexer_struct`.
+        an existing `lexer_matchers`.
         """
-        self.lexer_struct = lexer_struct
+        self.lexer_matchers = lexer_matchers
 
-    def get_lexer_struct(self):
+    def get_lexer_matchers(self):
         """Fetch the lexer struct for this dialect."""
-        if self.lexer_struct:
-            return self.lexer_struct
+        if self.lexer_matchers:
+            return self.lexer_matchers
         else:
-            raise ValueError(
-                "Lexing struct has not been set for dialect {0}".format(self)
-            )
+            raise ValueError(f"Lexing struct has not been set for dialect {self}")
 
-    def patch_lexer_struct(self, lexer_patch):
+    def patch_lexer_matchers(self, lexer_patch):
         """Patch an existing lexer struct.
 
         Used to edit the lexer of a sub-dialect.
         """
         buff = []
-        if not self.lexer_struct:
+        if not self.lexer_matchers:
             raise ValueError("Lexer struct must be defined before it can be patched!")
 
         # Make a new data struct for lookups
-        patch_dict = {elem[0]: elem for elem in lexer_patch}
+        patch_dict = {elem.name: elem for elem in lexer_patch}
 
-        for elem in self.lexer_struct:
-            if elem[0] in patch_dict:
-                buff.append(patch_dict[elem[0]])
+        for elem in self.lexer_matchers:
+            if elem.name in patch_dict:
+                buff.append(patch_dict[elem.name])
             else:
                 buff.append(elem)
         # Overwrite with the buffer once we're done
-        self.lexer_struct = buff
+        self.lexer_matchers = buff
 
-    def insert_lexer_struct(self, lexer_patch, before):
+    def insert_lexer_matchers(self, lexer_patch, before):
         """Insert new records into an existing lexer struct.
 
         Used to edit the lexer of a sub-dialect. The patch is
@@ -226,11 +280,11 @@ class Dialect:
         """
         buff = []
         found = False
-        if not self.lexer_struct:
+        if not self.lexer_matchers:
             raise ValueError("Lexer struct must be defined before it can be patched!")
 
-        for elem in self.lexer_struct:
-            if elem[0] == before:
+        for elem in self.lexer_matchers:
+            if elem.name == before:
                 found = True
                 for patch in lexer_patch:
                     buff.append(patch)
@@ -243,7 +297,7 @@ class Dialect:
                 "Lexer struct insert before '%s' failed because tag never found."
             )
         # Overwrite with the buffer once we're done
-        self.lexer_struct = buff
+        self.lexer_matchers = buff
 
     def get_root_segment(self):
         """Get the root segment of the dialect."""
