@@ -10,9 +10,24 @@ class SQLBaseError(ValueError):
     _code: Optional[str] = None
     _identifier = "base"
 
-    def __init__(self, *args, **kwargs):
-        self.ignore = kwargs.pop("ignore", False)
-        super(SQLBaseError, self).__init__(*args, **kwargs)
+    def __init__(
+        self,
+        *args,
+        pos=None,
+        line_no=0,
+        line_pos=0,
+        ignore=False,
+        fatal=False,
+        **kwargs
+    ):
+        self.fatal = fatal
+        self.ignore = ignore
+        if pos:
+            self.line_no, self.line_pos = pos.source_position()
+        else:
+            self.line_no = line_no
+            self.line_pos = line_pos
+        super().__init__(*args, **kwargs)
 
     @property
     def fixable(self):
@@ -54,67 +69,17 @@ class SQLBaseError(ValueError):
             else:
                 return self.__class__.__name__
 
-    def line_no(self):
-        """Return the line number of the violation."""
-        pm = self.pos_marker()
-        if pm:
-            return pm.line_no
-        else:
-            return None
-
-    def line_pos(self):
-        """Return the line position of the violation."""
-        pm = self.pos_marker()
-        if pm:
-            return pm.line_pos
-        else:
-            return None
-
-    def char_pos(self):
-        """Return the character position in file of the violation."""
-        pm = self.pos_marker()
-        if pm:
-            return pm.char_pos
-        else:
-            return 0
-
-    def pos_marker(self):
-        """Get the position marker of the violation.
-
-        Returns:
-            The :obj:`PosMarker` of the segments if the violation has a segment,
-            the :obj:`PosMarker` directly stored in a `pos` attribute or None
-            if neither a present.
-
-        """
-        if hasattr(self, "segment"):
-            # Linting and Parsing Errors
-            return self.segment.pos_marker
-        elif hasattr(self, "pos"):
-            # Lexing errors
-            return self.pos
-        else:
-            return None
-
-    def get_info_tuple(self):
-        """Get a tuple representation of this violation.
-
-        Returns:
-            A `tuple` of (code, line_no, line_pos, description)
-
-        """
-        return self.rule_code(), self.line_no(), self.line_pos(), self.desc()
-
     def get_info_dict(self):
-        """Get a dictionary representation of this violation.
-
-        Returns:
-            A `dictionary` with keys (code, line_no, line_pos, description)
+        """Return a dict of properties.
 
+        This is useful in the API for outputting violations.
         """
-        return dict(
-            zip(("code", "line_no", "line_pos", "description"), self.get_info_tuple())
-        )
+        return {
+            "line_no": self.line_no,
+            "line_pos": self.line_pos,
+            "code": self.rule_code(),
+            "description": self.desc(),
+        }
 
     def ignore_if_in(self, ignore_iterable):
         """Ignore this violation if it matches the iterable."""
@@ -138,9 +103,11 @@ class SQLTemplaterError(SQLBaseError):
     _code = "TMP"
     _identifier = "templating"
 
-    def __init__(self, *args, **kwargs):
-        self.pos = kwargs.pop("pos", None)
-        super(SQLTemplaterError, self).__init__(*args, **kwargs)
+
+class SQLTemplaterSkipFile(RuntimeError):
+    """An error returned from a templater to skip a file."""
+
+    pass
 
 
 class SQLLexError(SQLBaseError):
@@ -155,11 +122,6 @@ class SQLLexError(SQLBaseError):
     _code = "LXR"
     _identifier = "lexing"
 
-    def __init__(self, *args, **kwargs):
-        # Store the segment on creation - we might need it later
-        self.pos = kwargs.pop("pos", None)
-        super(SQLLexError, self).__init__(*args, **kwargs)
-
 
 class SQLParseError(SQLBaseError):
     """An error which occurred during parsing.
@@ -175,10 +137,12 @@ class SQLParseError(SQLBaseError):
     _code = "PRS"
     _identifier = "parsing"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, segment=None, **kwargs):
         # Store the segment on creation - we might need it later
-        self.segment = kwargs.pop("segment", None)
-        super(SQLParseError, self).__init__(*args, **kwargs)
+        self.segment = segment
+        if self.segment:
+            kwargs["pos"] = self.segment.pos_marker
+        super().__init__(*args, **kwargs)
 
 
 class SQLLintError(SQLBaseError):
@@ -197,13 +161,17 @@ class SQLLintError(SQLBaseError):
 
     _identifier = "linting"
 
-    def __init__(self, *args, **kwargs):
+    def __init__(
+        self, *args, segment=None, rule=None, fixes=None, description=None, **kwargs
+    ):
         # Something about position, message and fix?
-        self.segment = kwargs.pop("segment", None)
-        self.rule = kwargs.pop("rule", None)
-        self.fixes = kwargs.pop("fixes", [])
-        self.description = kwargs.pop("description", None)
-        super(SQLLintError, self).__init__(*args, **kwargs)
+        self.segment = segment
+        if self.segment:
+            kwargs["pos"] = self.segment.pos_marker
+        self.rule = rule
+        self.fixes = fixes or []
+        self.description = description
+        super().__init__(*args, **kwargs)
 
     @property
     def fixable(self):
@@ -216,13 +184,14 @@ class SQLLintError(SQLBaseError):
         """Get a tuple representing this error. Mostly for testing."""
         return (
             self.rule.code,
-            self.segment.pos_marker.line_no,
-            self.segment.pos_marker.line_pos,
+            self.line_no,
+            self.line_pos,
         )
 
     def __repr__(self):
-        return (
-            "<SQLLintError: rule {0} pos:{1!r}, #fixes: {2}, description: {3}>".format(
-                self.rule_code(), self.pos_marker(), len(self.fixes), self.description
-            )
+        return "<SQLLintError: rule {} pos:{!r}, #fixes: {}, description: {}>".format(
+            self.rule_code(),
+            (self.line_no, self.line_pos),
+            len(self.fixes),
+            self.description,
         )
