@@ -5,10 +5,13 @@
 from sqlfluff.core.parser.segments import (
     BaseSegment,
     RawSegment,
+    CodeSegment,
+    UnlexableSegment,
+    CommentSegment,
+    WhitespaceSegment,
+    NewlineSegment,
     KeywordSegment,
     SymbolSegment,
-    ReSegment,
-    NamedSegment,
     Indent,
     Dedent,
     SegmentGenerator,
@@ -24,8 +27,11 @@ from sqlfluff.core.parser.grammar import (
     Ref,
     Anything,
     Nothing,
+    OptionallyBracketed,
+    Conditional,
 )
-from sqlfluff.core.parser.markers import FilePositionMarker
-from sqlfluff.core.parser.lexer import Lexer
+from sqlfluff.core.parser.parsers import StringParser, NamedParser, RegexParser
+from sqlfluff.core.parser.markers import PositionMarker
+from sqlfluff.core.parser.lexer import Lexer, StringLexer, RegexLexer
 from sqlfluff.core.parser.parser import Parser
 from sqlfluff.core.parser.matchable import Matchable
