@@ -1,6 +1,6 @@
 """Defines the Parser class."""
 
-from typing import Optional, Tuple, TYPE_CHECKING
+from typing import Optional, Sequence, TYPE_CHECKING
 
 from sqlfluff.core.parser.context import RootParseContext
 from sqlfluff.core.config import FluffConfig
@@ -19,10 +19,12 @@ class Parser:
         self.config = FluffConfig.from_kwargs(config=config, dialect=dialect)
         self.RootSegment = self.config.get("dialect_obj").get_root_segment()
 
-    def parse(self, segments: Tuple["BaseSegment", ...], recurse=True) -> "BaseSegment":
+    def parse(
+        self, segments: Sequence["BaseSegment"], recurse=True
+    ) -> Optional["BaseSegment"]:
         """Parse a series of lexed tokens using the current dialect."""
         if not segments:
-            raise ValueError("Cannot parse an empty iterable of segments.")
+            return None
         # Instantiate the root segment
         root_segment = self.RootSegment(segments=segments)
         # Call .parse() on that segment
