@@ -9,10 +9,8 @@ Here we define:
 """
 
 from io import StringIO
-import copy
-from benchit import BenchIt
 from cached_property import cached_property
-from typing import Optional, List, Tuple, NamedTuple, Iterator
+from typing import Any, Callable, Optional, List, Tuple, NamedTuple, Iterator
 import logging
 
 from sqlfluff.core.string_helpers import (
@@ -28,7 +26,7 @@ from sqlfluff.core.parser.helpers import (
     trim_non_code_segments,
 )
 from sqlfluff.core.parser.matchable import Matchable
-from sqlfluff.core.parser.markers import EnrichedFilePositionMarker
+from sqlfluff.core.parser.markers import PositionMarker
 from sqlfluff.core.parser.context import ParseContext
 
 # Instantiate the linter logger (only for use in methods involved with fixing.)
@@ -40,10 +38,10 @@ class FixPatch(NamedTuple):
 
     templated_slice: slice
     fixed_raw: str
-    # The patch type, functions mostly for debugging and explanation
+    # The patch category, functions mostly for debugging and explanation
     # than for function. It allows traceability of *why* this patch was
-    # generated.
-    patch_type: str
+    # generated. It has no siginificance for processing.
+    patch_category: str
 
 
 class BaseSegment:
@@ -67,28 +65,28 @@ class BaseSegment:
     # `type` should be the *category* of this kind of segment
     type = "base"
     parse_grammar: Optional[Matchable] = None
-    match_grammar: Optional[Matchable] = None
+    # We define the type here but no value. Subclasses must provide a value.
+    match_grammar: Matchable
     comment_seperate = False
-    is_whitespace = False
     optional = False  # NB: See the sequence grammar for details
-    is_segment = True
-    _name = None
+    _name: Optional[str] = None
     is_meta = False
     # Are we able to have non-code at the start or end?
     can_start_end_non_code = False
     # Can we allow it to be empty? Usually used in combination
     # with the can_start_end_non_code.
     allow_empty = False
-    # What should we trim off the ends to get to content
-    trim_chars = None
-    trim_start = None
-    # A cache variable for expandable
-    _is_expandable = None
+    # What other kwargs need to be copied when applying fixes.
+    additional_kwargs: List[str] = []
 
-    def __init__(self, segments, pos_marker=None, validate=True):
+    def __init__(self, segments, pos_marker=None, name: Optional[str] = None):
+        # A cache variable for expandable
+        self._is_expandable = None
+        # Surrogate name option.
+        self._surrogate_name = name
         if len(segments) == 0:
             raise RuntimeError(
-                "Setting {0} with a zero length segment set. This shouldn't happen.".format(
+                "Setting {} with a zero length segment set. This shouldn't happen.".format(
                     self.__class__
                 )
             )
@@ -101,60 +99,52 @@ class BaseSegment:
         elif isinstance(segments, list):
             self.segments = tuple(segments)
         else:
-            raise TypeError(
-                "Unexpected type passed to BaseSegment: {0}".format(type(segments))
-            )
+            raise TypeError(f"Unexpected type passed to BaseSegment: {type(segments)}")
 
-        # Check elements of segments:
-        self.validate_segments(validate=validate)
-
-        if pos_marker:
-            self.pos_marker = pos_marker
-        else:
+        if not pos_marker:
             # If no pos given, it's the pos of the first segment.
             if isinstance(segments, (tuple, list)):
-                # Find the first segment with an enriched position marker
-                first_enriched = next(
-                    (
-                        seg.pos_marker
-                        for seg in segments
-                        if isinstance(seg.pos_marker, EnrichedFilePositionMarker)
-                    ),
-                    # Default to the first un-enriched segment
-                    segments[0].pos_marker,
-                )
-                self.pos_marker = first_enriched.combine(
+                pos_marker = PositionMarker.from_child_markers(
                     *(seg.pos_marker for seg in segments)
                 )
             else:
                 raise TypeError(
-                    "Unexpected type passed to BaseSegment: {0}".format(type(segments))
+                    f"Unexpected type passed to BaseSegment: {type(segments)}"
                 )
+        self.pos_marker: PositionMarker = pos_marker
 
     def __eq__(self, other):
-        # Equal if type, content and pos are the same
         # NB: this should also work for RawSegment
         return (
             # Same class NAME. (could be constructed elsewhere)
             self.__class__.__name__ == other.__class__.__name__
             and (self.raw == other.raw)
-            and (self.pos_marker == other.pos_marker)
+            # Both must have a non-null position marker to compare.
+            and self.pos_marker
+            and other.pos_marker
+            # We only match that the *start* is the same. This means we can
+            # still effectively construct searches look for segments.
+            # This is important for .apply_fixes().
+            and (
+                self.pos_marker.start_point_marker()
+                == other.pos_marker.start_point_marker()
+            )
         )
 
     def __repr__(self):
-        return "<{0}: ({1})>".format(self.__class__.__name__, self.pos_marker)
+        return f"<{self.__class__.__name__}: ({self.pos_marker})>"
 
     # ################ PRIVATE PROPERTIES
 
     @property
     def _comments(self):
         """Returns only the comment elements of this segment."""
-        return [seg for seg in self.segments if seg.type == "comment"]
+        return [seg for seg in self.segments if seg.is_type("comment")]
 
     @property
     def _non_comments(self):
         """Returns only the non-comment elements of this segment."""
-        return [seg for seg in self.segments if seg.type != "comment"]
+        return [seg for seg in self.segments if not seg.is_type("comment")]
 
     # ################ PUBLIC PROPERTIES
 
@@ -162,16 +152,17 @@ class BaseSegment:
     def name(self):
         """The name of this segment.
 
-        The reason for two routes for names is that some subclasses
-        might want to override the name rather than just getting it
-        the class name.
+        The reason for three routes for names is that some subclasses
+        might want to override the name rather than just getting
+        the class name. Instances may also override this with the
+        _surrogate_name.
 
         Name should be specific to this kind of segment, while `type`
         should be a higher level descriptor of the kind of segment.
         For example, the name of `+` is 'plus' but the type might be
         'binary_operator'.
         """
-        return self._name or self.__class__.__name__
+        return self._surrogate_name or self._name or self.__class__.__name__
 
     @property
     def is_expandable(self):
@@ -204,6 +195,11 @@ class BaseSegment:
         """Return True if this is entirely made of comments."""
         return all(seg.is_comment for seg in self.segments)
 
+    @cached_property
+    def is_whitespace(self):
+        """Return True if this segment is entirely whitespace."""
+        return all(seg.is_whitespace for seg in self.segments)
+
     @cached_property
     def raw(self):
         """Make a string from the segments of this segment."""
@@ -249,7 +245,6 @@ class BaseSegment:
                     segs += (stmt,)
                     continue
             except Exception as err:
-                # raise ValueError("{0} has no attribute `is_expandable`. This segment appears poorly constructed.".format(stmt))
                 parse_context.logger.error(
                     "%s has no attribute `is_expandable`. This segment appears poorly constructed.",
                     stmt,
@@ -257,11 +252,11 @@ class BaseSegment:
                 raise err
             if not hasattr(stmt, "parse"):
                 raise ValueError(
-                    "{0} has no method `parse`. This segment appears poorly constructed.".format(
+                    "{} has no method `parse`. This segment appears poorly constructed.".format(
                         stmt
                     )
                 )
-            parse_depth_msg = "Parse Depth {0}. Expanding: {1}: {2!r}".format(
+            parse_depth_msg = "Parse Depth {}. Expanding: {}: {!r}".format(
                 parse_context.parse_depth,
                 stmt.__class__.__name__,
                 curtail_string(stmt.raw, length=40),
@@ -277,52 +272,77 @@ class BaseSegment:
         check_still_complete(segments, segs, ())
         return segs
 
-    @staticmethod
-    def _realign_segments(segments, starting_pos=None, meta_only=False):
-        """Realign the positions in the provided segments."""
-        seg_buffer = []
-        todo_buffer = list(segments)
-        if not todo_buffer:
-            return ()
-        # If starting pos not provided, take it from the first of the buffer.
-        running_pos = starting_pos
-        if not running_pos:
-            for seg in todo_buffer:
-                if not seg.is_meta:
-                    running_pos = seg.pos_marker
+    @classmethod
+    def _position_segments(cls, segments, parent_pos=None):
+        """Refresh positions of segments within a span.
+
+        This does two things:
+        - Assign positions to any segments without them.
+        - Updates the working line_no and line_pos for all
+          segments during fixing.
+
+        New segments are assumed to be metas or insertions
+        and so therefore have a zero-length position in the
+        source and templated file.
+        """
+        # If there are no segments, there's no need to reposition.
+        if not segments:
+            return segments
+
+        # Work out our starting position for working through
+        if parent_pos:
+            line_no = parent_pos.working_line_no
+            line_pos = parent_pos.working_line_pos
+        # If we don't have it, infer it from the first position
+        # in this segment that does have a position.
+        else:
+            for fwd_seg in segments:
+                if fwd_seg.pos_marker:
+                    line_no = fwd_seg.pos_marker.working_line_no
+                    line_pos = fwd_seg.pos_marker.working_line_pos
                     break
             else:
-                raise ValueError("No starting pos provided and unable to infer.")
+                linter_logger.warning("SEG: %r, POS: %r", segments, parent_pos)
+                raise ValueError("Unable to find working position.")
 
-        # Strip the starting position regardless. This means
-        # we don't accidentally contaminate any inserted initial
-        # segments.
-        running_pos = running_pos.strip()
+        # Use the index so that we can look forward
+        # and backward.
+        for idx, segment in enumerate(segments):
+            # Fill any that don't have a position.
+            if not segment.pos_marker:
+                # Can we get a position from the previous?
+                if idx > 0:
+                    segment.pos_marker = segments[idx - 1].pos_marker.end_point_marker()
+                # Can we get it from the parent?
+                elif parent_pos:
+                    segment.pos_marker = parent_pos.start_point_marker()
+                # Search forward for a following one, if we have to?
+                else:
+                    for fwd_seg in segments[idx + 1 :]:
+                        if fwd_seg.pos_marker:
+                            segments[
+                                idx
+                            ].pos_marker = fwd_seg.pos_marker.start_point_marker()
+                            break
+                    else:
+                        raise ValueError("Unable to position new segment")
 
-        while len(todo_buffer) > 0:
-            # Get the first off the buffer
-            seg = todo_buffer.pop(0)
+            # Update the working position.
+            segment.pos_marker = segment.pos_marker.with_working_position(
+                line_no,
+                line_pos,
+            )
+            line_no, line_pos = segment.pos_marker.infer_next_position(
+                segment.raw, line_no, line_pos
+            )
 
-            # We'll preserve statement indexes so we should keep track of that.
-            # When recreating, we use the DELTA of the index so that's what matter...
-            idx = seg.pos_marker.statement_index - running_pos.statement_index
-            new_pos = seg.pos_marker.shift_to(running_pos)
+            # If this segment has children, recurse and reposition them too.
+            if segment.segments:
+                segment.segments = cls._position_segments(
+                    segment.segments, parent_pos=segment.pos_marker
+                )
 
-            if not meta_only or seg.is_meta:
-                # Copy the segment
-                seg_copy = copy.copy(seg)
-                # Update the position
-                seg_copy.pos_marker = new_pos
-                # Realign the children of that class if required.
-                if len(seg_copy.segments) > 0:
-                    seg_copy = seg_copy.realign()
-                seg = seg_copy
-
-            # Update the running position with the content of that segment.
-            running_pos = running_pos.advance_by(raw=seg.raw, idx=idx)
-            # Add the buffer to my new segment
-            seg_buffer.append(seg)
-        return tuple(seg_buffer)
+        return segments
 
     # ################ CLASS METHODS
 
@@ -351,17 +371,20 @@ class BaseSegment:
         return cls.optional
 
     @classmethod
-    def is_type(cls, *seg_type):
-        """Is this segment (or its parent) of the given type."""
+    def class_is_type(cls, *seg_type):
+        """Is this segment class (or its parent) of the given type."""
         # Do we match on the type of _this_ class.
         if cls.type in seg_type:
             return True
-        # Have we reached the bottom?
-        elif cls.type == "base":
-            return False
-        # If not, check parent classes.
-        else:
-            return any(base_class.is_type(*seg_type) for base_class in cls.__bases__)
+        # If not, check types of parents.
+        for base_class in cls.__bases__:
+            if base_class is object:
+                break
+            elif base_class.type in seg_type:
+                return True
+            elif base_class.type == "base":
+                break
+        return False
 
     @classmethod
     def structural_simplify(cls, elem):
@@ -432,7 +455,7 @@ class BaseSegment:
             # Calling unify here, allows the MatchResult class to do all the type checking.
             if not isinstance(m, MatchResult):
                 raise TypeError(
-                    "[PD:{0} MD:{1}] {2}.match. Result is {3}, not a MatchResult!".format(
+                    "[PD:{} MD:{}] {}.match. Result is {}, not a MatchResult!".format(
                         parse_context.parse_depth,
                         parse_context.match_depth,
                         cls.__name__,
@@ -448,7 +471,7 @@ class BaseSegment:
                 return MatchResult.from_unmatched(segments)
         else:
             raise NotImplementedError(
-                "{0} has no match function implemented".format(cls.__name__)
+                f"{cls.__name__} has no match function implemented"
             )
 
     # ################ PRIVATE INSTANCE METHODS
@@ -462,7 +485,7 @@ class BaseSegment:
         padded_type = "{padding}{modifier}{type}".format(
             padding=" " * (ident * tabsize),
             modifier="[META] " if self.is_meta else "",
-            type=self.type + ":",
+            type=self.get_type() + ":",
         )
         preface = "{pos:20}|{padded_type:60}  {suffix}".format(
             pos=str(self.pos_marker) if self.pos_marker else "-",
@@ -474,6 +497,14 @@ class BaseSegment:
 
     # ################ PUBLIC INSTANCE METHODS
 
+    def get_type(self):
+        """Returns the type of this segment as a string."""
+        return self.type
+
+    def is_type(self, *seg_type):
+        """Is this segment (or its parent) of the given type."""
+        return self.class_is_type(*seg_type)
+
     def invalidate_caches(self):
         """Invalidate the cached properties.
 
@@ -483,53 +514,23 @@ class BaseSegment:
         for key in ["is_code", "is_comment", "raw", "raw_upper", "matched_length"]:
             self.__dict__.pop(key, None)
 
-    def validate_segments(self, text="constructing", validate=True):
-        """Validate the current set of segments.
+    def get_start_point_marker(self):
+        """Get a point marker at the start of this segment."""
+        return self.pos_marker.start_point_marker()
 
-        Check the elements of the `segments` attribute are all
-        themselves segments, and that the positions match up.
+    def get_end_point_marker(self):
+        """Get a point marker at the end of this segment."""
+        return self.pos_marker.end_point_marker()
 
-        `validate` confirms whether we should check contiguousness.
-        """
-        # Placeholder variables for positions
-        start_pos = None
-        end_pos = None
-        prev_seg = None
-        for elem in self.segments:
-            if not isinstance(elem, BaseSegment):
-                raise TypeError(
-                    "In {0} {1}, found an element of the segments tuple which"
-                    " isn't a segment. Instead found element of type {2}.\nFound: {3}\nFull segments:{4}".format(
-                        text, type(self), type(elem), elem, self.segments
-                    )
-                )
-            # While applying fixes, we shouldn't validate here, because it will fail.
-            if validate:
-                # If we have a comparison point, validate that
-                if end_pos and elem.get_start_pos_marker() != end_pos:
-                    raise TypeError(
-                        "In {0} {1}, found an element of the segments tuple which"
-                        " isn't contiguous with previous: {2} > {3}. End pos: {4}."
-                        " Prev String: {5!r}".format(
-                            text, type(self), prev_seg, elem, end_pos, prev_seg.raw
-                        )
-                    )
-                start_pos = elem.get_start_pos_marker()
-                end_pos = elem.get_end_pos_marker()
-                prev_seg = elem
-                if start_pos.advance_by(elem.raw) != end_pos:
-                    raise TypeError(
-                        "In {0} {1}, found an element of the segments tuple which"
-                        " isn't self consistent: {2}".format(text, type(self), elem)
-                    )
+    def get_start_loc(self):
+        """Get a location tuple at the start of this segment."""
+        return self.pos_marker.working_loc
 
-    def get_end_pos_marker(self):
-        """Return the pos marker at the end of this segment."""
-        return self.segments[-1].get_end_pos_marker()
-
-    def get_start_pos_marker(self):
-        """Return the pos marker at the start of this segment."""
-        return self.segments[0].get_start_pos_marker()
+    def get_end_loc(self):
+        """Get a location tuple at the end of this segment."""
+        return self.pos_marker.working_loc_after(
+            self.raw,
+        )
 
     def stringify(self, ident=0, tabsize=4, code_only=False):
         """Use indentation to render this segment and its children as a string."""
@@ -570,32 +571,36 @@ class BaseSegment:
                     )
         return buff.getvalue()
 
-    def to_tuple(self, **kwargs):
-        """Return a tuple structure from this segment.
-
-        NB: If he segment is a meta segment, i.e. it's an indent or dedent,
-        then it will never be returned from here!
-        """
+    def to_tuple(self, code_only=False, show_raw=False, include_meta=False):
+        """Return a tuple structure from this segment."""
         # works for both base and raw
-        code_only = kwargs.get("code_only", False)
-        show_raw = kwargs.get("show_raw", False)
 
         if show_raw and not self.segments:
-            result = (self.type, self.raw)
+            result = (self.get_type(), self.raw)
         elif code_only:
             result = (
-                self.type,
+                self.get_type(),
                 tuple(
-                    seg.to_tuple(**kwargs)
+                    seg.to_tuple(
+                        code_only=code_only,
+                        show_raw=show_raw,
+                        include_meta=include_meta,
+                    )
                     for seg in self.segments
                     if seg.is_code and not seg.is_meta
                 ),
             )
         else:
             result = (
-                self.type,
+                self.get_type(),
                 tuple(
-                    seg.to_tuple(**kwargs) for seg in self.segments if not seg.is_meta
+                    seg.to_tuple(
+                        code_only=code_only,
+                        show_raw=show_raw,
+                        include_meta=include_meta,
+                    )
+                    for seg in self.segments
+                    if include_meta or not seg.is_meta
                 ),
             )
         return result
@@ -661,19 +666,47 @@ class BaseSegment:
                 buff.append(seg)
         return buff
 
-    def recursive_crawl(self, *seg_type):
+    def select_children(
+        self,
+        start_seg: Optional["BaseSegment"] = None,
+        stop_seg: Optional["BaseSegment"] = None,
+        select_if: Optional[Callable[["BaseSegment"], Any]] = None,
+        loop_while: Optional[Callable[["BaseSegment"], Any]] = None,
+    ):
+        """Retrieve subset of children based on range and filters.
+
+        Often useful by linter rules when generating fixes, e.g. to find
+        whitespace segments between two already known segments.
+        """
+        start_index = self.segments.index(start_seg) if start_seg else -1
+        stop_index = self.segments.index(stop_seg) if stop_seg else len(self.segments)
+        buff = []
+        for seg in self.segments[start_index + 1 : stop_index]:
+            if loop_while and not loop_while(seg):
+                break
+            if not select_if or select_if(seg):
+                buff.append(seg)
+        return buff
+
+    def recursive_crawl(self, *seg_type, recurse_into=True):
         """Recursively crawl for segments of a given type.
 
         Args:
             seg_type: :obj:`str`: one or more type of segment
                 to look for.
+            recurse_into: :obj:`bool`: When an element of type "seg_type" is
+                found, whether to recurse into it.
         """
         # Check this segment
         if self.is_type(*seg_type):
+            match = True
             yield self
-        # Recurse
-        for seg in self.segments:
-            yield from seg.recursive_crawl(*seg_type)
+        else:
+            match = False
+        if recurse_into or not match:
+            # Recurse
+            for seg in self.segments:
+                yield from seg.recursive_crawl(*seg_type, recurse_into=recurse_into)
 
     def path_to(self, other):
         """Given a segment which is assumed within self, get the intermediate segments.
@@ -688,11 +721,8 @@ class BaseSegment:
             return [self]
 
         # Are we in the right ballpark?
-        if (
-            not self.get_start_pos_marker()
-            <= other.get_start_pos_marker()
-            <= self.get_end_pos_marker()
-        ):
+        # NB: Comparisons have a higher precedence than `not`.
+        if not self.get_start_loc() <= other.get_start_loc() <= self.get_end_loc():
             return None
 
         # Do we have any child segments at all?
@@ -706,13 +736,17 @@ class BaseSegment:
                 return [self] + res
         return None
 
-    def parse(self, parse_context=None):
+    def parse(self, parse_context=None, parse_grammar=None):
         """Use the parse grammar to find subsegments within this segment.
 
         A large chunk of the logic around this can be found in the `expand` method.
 
         Use the parse setting in the context for testing, mostly to check how deep to go.
         True/False for yes or no, an integer allows a certain number of levels.
+
+        Optionally, this method allows a custom parse grammar to be
+        provided which will override any existing parse grammar
+        on the segment.
         """
         # Clear the blacklist cache so avoid missteps
         if parse_context:
@@ -724,10 +758,11 @@ class BaseSegment:
             return self
 
         # Check the Parse Grammar
-        if self.parse_grammar is None:
+        parse_grammar = parse_grammar or self.parse_grammar
+        if parse_grammar is None:
             # No parse grammar, go straight to expansion
             parse_context.logger.debug(
-                "{0}.parse: no grammar. Going straight to expansion".format(
+                "{}.parse: no grammar. Going straight to expansion".format(
                     self.__class__.__name__
                 )
             )
@@ -743,24 +778,24 @@ class BaseSegment:
                 post_nc = ()
                 if (not segments[0].is_code) and (not segments[0].is_meta):
                     raise ValueError(
-                        "Segment {0} starts with non code segment: {1!r}.\n{2!r}".format(
+                        "Segment {} starts with non code segment: {!r}.\n{!r}".format(
                             self, segments[0].raw, segments
                         )
                     )
                 if (not segments[-1].is_code) and (not segments[-1].is_meta):
                     raise ValueError(
-                        "Segment {0} ends with non code segment: {1!r}.\n{2!r}".format(
+                        "Segment {} ends with non code segment: {!r}.\n{!r}".format(
                             self, segments[-1].raw, segments
                         )
                     )
 
             # NOTE: No match_depth kwarg, because this is the start of the matching.
             with parse_context.matching_segment(self.__class__.__name__) as ctx:
-                m = self.parse_grammar.match(segments=segments, parse_context=ctx)
+                m = parse_grammar.match(segments=segments, parse_context=ctx)
 
             if not isinstance(m, MatchResult):
                 raise TypeError(
-                    "[PD:{0}] {1}.match. Result is {2}, not a MatchResult!".format(
+                    "[PD:{}] {}.match. Result is {}, not a MatchResult!".format(
                         parse_context.parse_depth, self.__class__.__name__, type(m)
                     )
                 )
@@ -798,33 +833,24 @@ class BaseSegment:
                     + (
                         UnparsableSegment(
                             segments=segments,
-                            expected=self.type,
+                            expected=self.name,
                         ),  # NB: tuple
                     )
                     + post_nc
                 )
-
-            # Validate new segments
-            self.validate_segments(text="parsing")
-
-        bencher = BenchIt()  # starts the timer
-        bencher("Parse complete of {0!r}".format(self.__class__.__name__))
-
         # Recurse if allowed (using the expand method to deal with the expansion)
         parse_context.logger.debug(
-            "{0}.parse: Done Parse. Plotting Recursion. Recurse={1!r}".format(
+            "{}.parse: Done Parse. Plotting Recursion. Recurse={!r}".format(
                 self.__class__.__name__, parse_context.recurse
             )
         )
-        parse_depth_msg = "###\n#\n# Beginning Parse Depth {0}: {1}\n#\n###\nInitial Structure:\n{2}".format(
+        parse_depth_msg = "###\n#\n# Beginning Parse Depth {}: {}\n#\n###\nInitial Structure:\n{}".format(
             parse_context.parse_depth + 1, self.__class__.__name__, self.stringify()
         )
         if parse_context.may_recurse():
             parse_context.logger.debug(parse_depth_msg)
             with parse_context.deeper_parse() as ctx:
                 self.segments = self.expand(self.segments, parse_context=ctx)
-        # Validate new segments
-        self.validate_segments(text="expanding")
 
         return self
 
@@ -857,7 +883,9 @@ class BaseSegment:
                     unused_fixes = []
                     while fix_buff:
                         f = fix_buff.pop()
-                        if f.anchor == seg:
+                        # Look for identity not just equality.
+                        # This handles potential positioning ambiguity.
+                        if f.anchor is seg:
                             linter_logger.debug(
                                 "Matched fix against segment: %s -> %s", f, seg
                             )
@@ -877,7 +905,7 @@ class BaseSegment:
                                     seg_buffer.append(seg)
                             else:
                                 raise ValueError(
-                                    "Unexpected edit_type: {0!r} in {1!r}".format(
+                                    "Unexpected edit_type: {!r} in {!r}".format(
                                         f.edit_type, f
                                     )
                                 )
@@ -903,36 +931,19 @@ class BaseSegment:
 
             # Reform into a new segment
             r = r.__class__(
-                segments=tuple(seg_buffer), pos_marker=r.pos_marker, validate=False
+                # Realign the segments within
+                segments=self._position_segments(
+                    tuple(seg_buffer), parent_pos=r.pos_marker
+                ),
+                pos_marker=r.pos_marker,
+                # Pass through any additional kwargs
+                **{k: getattr(self, k) for k in self.additional_kwargs},
             )
-
-            # Lastly, before returning, we should realign positions.
-            # Note: Realign also returns a copy
-            return r.realign(), fixes
+            # Return the new segment with any unused fixes.
+            return r, fixes
         else:
             return self, fixes
 
-    def realign(self):
-        """Realign the positions in this segment.
-
-        Returns:
-            a copy of this class with the pos_markers realigned.
-
-        Note: this is used mostly during fixes.
-
-        Realign is recursive. We will assume that the pos_marker of THIS segment is
-        truthful, and that during recursion it will have been set by the parent.
-
-        This function will align the pos marker if its direct children, we then
-        recurse to realign their children.
-
-        """
-        # Create a new version of this class with the new details
-        return self.__class__(
-            segments=self._realign_segments(self.segments, self.pos_marker),
-            pos_marker=self.pos_marker,
-        )
-
     def iter_patches(self, templated_str: str) -> Iterator[FixPatch]:
         """Iterate through the segments generating fix patches.
 
@@ -951,10 +962,10 @@ class BaseSegment:
         # If we're here, the segment doesn't match the original.
 
         # If it's all literal, then we don't need to recurse.
-        if self.pos_marker.is_literal:
+        if self.pos_marker.is_literal():
             # Yield the position in the source file and the patch
             yield FixPatch(
-                self.pos_marker.templated_slice, self.raw, patch_type="literal"
+                self.pos_marker.templated_slice, self.raw, patch_category="literal"
             )
         # Can we go deeper?
         elif not self.segments:
@@ -971,8 +982,8 @@ class BaseSegment:
             for seg_idx, segment in enumerate(self.segments):
 
                 # First check for insertions.
-                # We know it is new if the position marker is NOT ENRICHED.
-                if not isinstance(segment.pos_marker, EnrichedFilePositionMarker):
+                # We know it's an insertion if it has length but not in the templated file.
+                if segment.raw and segment.pos_marker.is_point():
                     # Add it to the insertion buffer if it has length:
                     if segment.raw:
                         insert_buff += segment.raw
@@ -997,7 +1008,7 @@ class BaseSegment:
                             segment.pos_marker.templated_slice.start,
                         ),
                         insert_buff,
-                        patch_type="mid_point",
+                        patch_category="mid_point",
                     )
                     insert_buff = ""
 
@@ -1018,10 +1029,61 @@ class BaseSegment:
                         self.pos_marker.templated_slice.stop,
                     ),
                     insert_buff,
-                    patch_type="end_point",
+                    patch_category="end_point",
                 )
 
 
+class BracketedSegment(BaseSegment):
+    """A segment containing a bracketed expression."""
+
+    type = "bracketed"
+    additional_kwargs = ["start_bracket", "end_bracket"]
+
+    def __init__(
+        self,
+        *args,
+        # These are tuples of segments but we're expecting them to
+        # be tuples of length 1. This is because we'll almost always
+        # be doing tuple arithmetic with the results and constructing
+        # 1-tuples on the fly is very easy to misread.
+        start_bracket: Tuple[BaseSegment] = None,
+        end_bracket: Tuple[BaseSegment] = None,
+        **kwargs,
+    ):
+        """Stash the bracket segments for later."""
+        if not start_bracket or not end_bracket:
+            raise ValueError(
+                "Attempted to construct Bracketed segment without specifying brackets."
+            )
+        self.start_bracket = start_bracket
+        self.end_bracket = end_bracket
+        super().__init__(*args, **kwargs)
+
+    @classmethod
+    def simple(cls, parse_context: ParseContext) -> Optional[List[str]]:
+        """Simple methods for bracketed and the persitent brackets."""
+        start_brackets = [
+            start_bracket
+            for _, start_bracket, _, persistent in parse_context.dialect.sets(
+                "bracket_pairs"
+            )
+            if persistent
+        ]
+        start_simple = []
+        for ref in start_brackets:
+            start_simple += parse_context.dialect.ref(ref).simple(parse_context)
+        return start_simple
+
+    @classmethod
+    def match(
+        cls, segments: Tuple["BaseSegment", ...], parse_context: ParseContext
+    ) -> MatchResult:
+        """Only useful as a terminator."""
+        if segments and isinstance(segments[0], cls):
+            return MatchResult((segments[0],), segments[1:])
+        return MatchResult.from_unmatched(segments)
+
+
 class UnparsableSegment(BaseSegment):
     """This is a segment which can't be parsed. It indicates a error during parsing."""
 
@@ -1039,7 +1101,7 @@ class UnparsableSegment(BaseSegment):
 
         NB Override this for specific subclasses if we want extra output.
         """
-        return "!! Expected: {0!r}".format(self._expected)
+        return f"!! Expected: {self._expected!r}"
 
     def iter_unparsables(self):
         """Iterate through any unparsables.
