@@ -1,15 +1,23 @@
 """A collection of decorators to modify rule docstrings for Sphinx."""
 
-from sqlfluff.core.rules.config_info import STANDARD_CONFIG_INFO_DICT
+from sqlfluff.core.rules.config_info import get_config_info
 from sqlfluff.core.rules.base import rules_logger  # noqa
 
 
+FIX_COMPATIBLE = "``sqlfluff fix`` compatible."
+
+
 def document_fix_compatible(cls):
     """Mark the rule as fixable in the documentation."""
-    cls.__doc__ = cls.__doc__.replace("\n", "\n\n``sqlfluff fix`` compatible.\n", 1)
+    cls.__doc__ = cls.__doc__.replace("\n", f"\n\n{FIX_COMPATIBLE}\n", 1)
     return cls
 
 
+def is_fix_compatible(cls) -> bool:
+    """Return whether the rule is documented as fixable."""
+    return FIX_COMPATIBLE in cls.__doc__
+
+
 def document_configuration(cls, ruleset="std"):
     """Add a 'Configuration' section to a Rule docstring.
 
@@ -20,7 +28,7 @@ def document_configuration(cls, ruleset="std"):
     options in the docs, from a single source of truth.
     """
     if ruleset == "std":
-        config_info = STANDARD_CONFIG_INFO_DICT
+        config_info = get_config_info()
     else:
         raise (
             NotImplementedError(
@@ -30,7 +38,7 @@ def document_configuration(cls, ruleset="std"):
 
     config_doc = "\n    | **Configuration**"
     try:
-        for keyword in cls.config_keywords:
+        for keyword in sorted(cls.config_keywords):
             try:
                 info_dict = config_info[keyword]
             except KeyError:
@@ -39,12 +47,14 @@ def document_configuration(cls, ruleset="std"):
                         keyword, cls.__name__
                     )
                 )
-            config_doc += "\n    |     `{0}`: {1}. Must be one of {2}.".format(
-                keyword, info_dict["definition"], info_dict["validation"]
+            config_doc += "\n    |     `{}`: {}.".format(
+                keyword, info_dict["definition"]
             )
+            if "validation" in info_dict:
+                config_doc += " Must be one of {}.".format(info_dict["validation"])
             config_doc += "\n    |"
     except AttributeError:
-        rules_logger.info("No config_keywords defined for {0}".format(cls.__name__))
+        rules_logger.info(f"No config_keywords defined for {cls.__name__}")
         return cls
     # Add final blank line
     config_doc += "\n"
