@@ -200,7 +200,7 @@ class PythonTemplater(RawTemplater):
         return live_context
 
     def process(
-        self, *, in_str: str, fname: Optional[str] = None, config=None
+        self, *, in_str: str, fname: str, config=None, formatter=None
     ) -> Tuple[Optional[TemplatedFile], list]:
         """Process a string and return a TemplatedFile.
 
@@ -218,6 +218,7 @@ class PythonTemplater(RawTemplater):
                 mostly for loading config files at runtime.
             config (:obj:`FluffConfig`): A specific config to use for this
                 templating operation. Only necessary for some templaters.
+            formatter (:obj:`CallbackFormatter`): Optional object for output.
 
         """
         live_context = self.get_context(fname=fname, config=config)
@@ -226,7 +227,7 @@ class PythonTemplater(RawTemplater):
         except KeyError as err:
             # TODO: Add a url here so people can get more help.
             raise SQLTemplaterError(
-                "Failure in Python templating: {0}. Have you configured your variables?".format(
+                "Failure in Python templating: {}. Have you configured your variables?".format(
                     err
                 )
             )
@@ -246,7 +247,10 @@ class PythonTemplater(RawTemplater):
 
     @classmethod
     def slice_file(
-        cls, raw_str: str, templated_str: str, config=None
+        cls,
+        raw_str: str,
+        templated_str: str,
+        config=None,
     ) -> Tuple[List[RawFileSlice], List[TemplatedFileSlice], str]:
         """Slice the file to determine regions where we can fix."""
         templater_logger.info("Slicing File Template")
@@ -254,6 +258,9 @@ class PythonTemplater(RawTemplater):
         templater_logger.debug("    Templated String: %r", templated_str)
         # Slice the raw file
         raw_sliced = list(cls._slice_template(raw_str))
+        templater_logger.debug("    Raw Sliced:")
+        for idx, raw_slice in enumerate(raw_sliced):
+            templater_logger.debug("        %s: %r", idx, raw_slice)
         # Find the literals
         literals = [
             raw_slice.raw
@@ -281,14 +288,18 @@ class PythonTemplater(RawTemplater):
                     templated_str,
                 )
             )
-            templater_logger.debug("    Split Sliced: %s", split_sliced)
+            templater_logger.debug("    Split Sliced:")
+            for idx, split_slice in enumerate(split_sliced):
+                templater_logger.debug("        %s: %r", idx, split_slice)
             # Deal with uniques and coalesce the rest
             sliced_file = list(
                 cls._split_uniques_coalesce_rest(
                     split_sliced, raw_occurrences, templated_occurances, templated_str
                 )
             )
-            templater_logger.debug("    Fully Sliced: %s", sliced_file)
+            templater_logger.debug("    Fully Sliced:")
+            for idx, templ_slice in enumerate(sliced_file):
+                templater_logger.debug("        %s: %r", idx, templ_slice)
             unwrap_wrapped = (
                 True
                 if config is None
@@ -427,21 +438,26 @@ class PythonTemplater(RawTemplater):
             if field_name:
                 constructed_token = "{{{field_name}{conv}{spec}}}".format(
                     field_name=field_name,
-                    conv="!{}".format(conversion) if conversion else "",
-                    spec=":{}".format(format_spec) if format_spec else "",
+                    conv=f"!{conversion}" if conversion else "",
+                    spec=f":{format_spec}" if format_spec else "",
                 )
                 yield RawFileSlice(constructed_token, "templated", in_idx)
                 in_idx += len(constructed_token)
 
-    @staticmethod
+    @classmethod
     def _split_invariants(
+        cls,
         raw_sliced: List[RawFileSlice],
         literals: List[str],
         raw_occurances: Dict[str, List[int]],
         templated_occurances: Dict[str, List[int]],
         templated_str: str,
     ) -> Iterator[IntermediateFileSlice]:
-        """Split a sliced file on its invariant literals."""
+        """Split a sliced file on its invariant literals.
+
+        We prioritise the _longest_ invariants first as they
+        are more likely to the the anchors.
+        """
         # Calculate invariants
         invariants = [
             literal
@@ -449,6 +465,30 @@ class PythonTemplater(RawTemplater):
             if len(raw_occurances[literal]) == 1
             and len(templated_occurances[literal]) == 1
         ]
+        # Work through the invariants and make sure they appear
+        # in order.
+        for linv in sorted(invariants, key=len, reverse=True):
+            # Any invariants which have templated positions, relative
+            # to source positions, which aren't in order, should be
+            # ignored.
+
+            # Is this one still relevant?
+            if linv not in invariants:
+                continue
+
+            source_pos, templ_pos = raw_occurances[linv], templated_occurances[linv]
+            # Copy the list before iterating because we're going to edit it.
+            for tinv in invariants.copy():
+                if tinv != linv:
+                    src_dir = source_pos > raw_occurances[tinv]
+                    tmp_dir = templ_pos > templated_occurances[tinv]
+                    # If it's not in the same direction in the source and template remove it.
+                    if src_dir != tmp_dir:
+                        templater_logger.debug(
+                            "          Invariant found out of order: %r", tinv
+                        )
+                        invariants.remove(tinv)
+
         # Set up some buffers
         buffer: List[RawFileSlice] = []
         idx: Optional[int] = None
@@ -508,6 +548,11 @@ class PythonTemplater(RawTemplater):
         """Coalesce to the priority type."""
         # Make a set of types
         types = {elem.slice_type for elem in elems}
+        # Replace block types with templated
+        for typ in list(types):
+            if typ.startswith("block_"):
+                types.remove(typ)
+                types.add("templated")
         # Take the easy route if they're all the same type
         if len(types) == 1:
             return types.pop()
@@ -516,9 +561,7 @@ class PythonTemplater(RawTemplater):
         for p in priority:
             if p in types:
                 return p
-        raise RuntimeError(
-            "Exhausted priorities in _coalesce_types! {0!r}".format(types)
-        )
+        raise RuntimeError(f"Exhausted priorities in _coalesce_types! {types!r}")
 
     @classmethod
     def _split_uniques_coalesce_rest(
@@ -551,6 +594,19 @@ class PythonTemplater(RawTemplater):
                 yield from tail_buffer
                 tail_buffer = []
 
+            # Check whether we're handling a zero length slice.
+            if (
+                int_file_slice.templated_slice.stop
+                - int_file_slice.templated_slice.start
+                == 0
+            ):
+                point_combo = int_file_slice.coalesce()
+                templater_logger.debug(
+                    "        Yielding Point Combination: %s", point_combo
+                )
+                yield point_combo
+                continue
+
             # Yield anything simple
             try:
                 simple_elem = int_file_slice.try_simple()
@@ -632,49 +688,109 @@ class PythonTemplater(RawTemplater):
                 # Yield the uniques and coalesce anything between.
                 bookmark_idx = 0
                 for idx, raw_slice in enumerate(int_file_slice.slice_buffer):
-                    # Is this one a unique?
-                    if raw_slice.raw in two_way_uniques:
+                    pos = 0
+                    unq: Optional[str] = None
+                    # Does this element contain one of our uniques? If so, where?
+                    for unique in two_way_uniques:
+                        if unique in raw_slice.raw:
+                            pos = raw_slice.raw.index(unique)
+                            unq = unique
+
+                    if unq:
+                        # Yes it does. Handle it.
+
+                        # Get the position of the unique section.
                         unique_position = (
-                            raw_occs[raw_slice.raw][0],
-                            templ_occs[raw_slice.raw][0],
+                            raw_occs[unq][0],
+                            templ_occs[unq][0],
                         )
-                        # Do we have anything before it to process?
+                        templater_logger.debug(
+                            "            Handling Unique: %r, %s, %s, %r",
+                            unq,
+                            pos,
+                            unique_position,
+                            raw_slice,
+                        )
+
+                        # Handle full slices up to this one
                         if idx > bookmark_idx:
                             # Recurse to deal with any loops separately
-                            sub_section = int_file_slice.slice_buffer[bookmark_idx:idx]
                             yield from cls._split_uniques_coalesce_rest(
                                 [
                                     IntermediateFileSlice(
                                         "compound",
                                         # slice up to this unique
-                                        slice(starts[0], unique_position[0]),
-                                        slice(starts[1], unique_position[1]),
-                                        sub_section,
+                                        slice(starts[0], unique_position[0] - pos),
+                                        slice(starts[1], unique_position[1] - pos),
+                                        int_file_slice.slice_buffer[bookmark_idx:idx],
                                     )
                                 ],
                                 raw_occs,
                                 templ_occs,
                                 templated_str,
                             )
-                        # Process the value itself, withe the new starts.
+
+                        # Handle any potential partial slice if we're part way through this one.
+                        if pos > 0:
+                            yield TemplatedFileSlice(
+                                raw_slice.slice_type,
+                                slice(unique_position[0] - pos, unique_position[0]),
+                                slice(unique_position[1] - pos, unique_position[1]),
+                            )
+
+                        # Handle the unique itself and update the bookmark
                         starts = (
-                            unique_position[0] + len(raw_slice.raw),
-                            unique_position[1] + len(raw_slice.raw),
+                            unique_position[0] + len(unq),
+                            unique_position[1] + len(unq),
                         )
                         yield TemplatedFileSlice(
                             raw_slice.slice_type,
-                            # It's a literal so use its length
                             slice(unique_position[0], starts[0]),
                             slice(unique_position[1], starts[1]),
                         )
                         # Move the bookmark after this position
                         bookmark_idx = idx + 1
-                # At the end of the loop deal with any hangover
+
+                        # Handle any remnant after the unique.
+                        if raw_slice.raw[pos + len(unq) :]:
+                            remnant_length = len(raw_slice.raw) - (len(unq) + pos)
+                            _starts = starts
+                            starts = (
+                                starts[0] + remnant_length,
+                                starts[1] + remnant_length,
+                            )
+                            yield TemplatedFileSlice(
+                                raw_slice.slice_type,
+                                slice(_starts[0], starts[0]),
+                                slice(_starts[1], starts[1]),
+                            )
+
+                if bookmark_idx == 0:
+                    # This is a SAFETY VALVE. In Theory we should never be here
+                    # and if we are it implies an error elsewhere. This clause
+                    # should stop any potential infinite recursion in its tracks
+                    # by simply classifying the whole of the current block as
+                    # templated and just stopping here.
+                    # Bugs triggering this eventuality have been observed in 0.4.0.
+                    templater_logger.info(
+                        "        Safety Value Info: %s, %r",
+                        two_way_uniques,
+                        templated_str[int_file_slice.templated_slice],
+                    )
+                    templater_logger.warning(
+                        "        Python templater safety value unexpectedly triggered. "
+                        "Please report your raw and compiled query on github for debugging."
+                    )
+                    # NOTE: If a bug is reported here, this will incorrectly
+                    # classify more of the query as "templated" than it should.
+                    yield coalesced
+                    continue
+
+                # At the end of the loop deal with any remaining slices.
+                # The above "Safety Valve"TM should keep us safe from infinite
+                # recursion.
                 if len(int_file_slice.slice_buffer) > bookmark_idx:
                     # Recurse to deal with any loops separately
-                    sub_section = int_file_slice.slice_buffer[
-                        bookmark_idx : len(int_file_slice.slice_buffer)
-                    ]
                     yield from cls._split_uniques_coalesce_rest(
                         [
                             IntermediateFileSlice(
@@ -682,7 +798,10 @@ class PythonTemplater(RawTemplater):
                                 # Slicing is easy here, we have no choice
                                 slice(starts[0], int_file_slice.source_slice.stop),
                                 slice(starts[1], int_file_slice.templated_slice.stop),
-                                sub_section,
+                                # Calculate the subsection to deal with.
+                                int_file_slice.slice_buffer[
+                                    bookmark_idx : len(int_file_slice.slice_buffer)
+                                ],
                             )
                         ],
                         raw_occs,
@@ -751,15 +870,15 @@ class PythonTemplater(RawTemplater):
                     # all as a tuple, because if we could do any better
                     # we would have done it by now.
 
-                    source_slice: Optional[slice] = None
+                    # Can we identify a meaningful portion of the patch
+                    # to recurse a split?
+                    sub_section: Optional[List[RawFileSlice]] = None
                     # If it's the start, the slicing is easy
                     if starts[1] == int_file_slice.templated_slice.stop:
-                        source_slice = slice(starts[0], raw_idx)
                         sub_section = int_file_slice.slice_buffer[:this_owu_idx]
                     # If we are AFTER the previous in the template, then it's
                     # also easy. [assuming it's not the same owu]
                     elif raw_idx > starts[0] and last_owu_idx != this_owu_idx:
-                        source_slice = slice(starts[0], raw_idx)
                         if last_owu_idx:
                             sub_section = int_file_slice.slice_buffer[
                                 last_owu_idx + 1 : this_owu_idx
@@ -769,7 +888,10 @@ class PythonTemplater(RawTemplater):
 
                     # If we succeeded in one of the above, we can also recurse
                     # and be more intelligent with the other sections.
-                    if source_slice:
+                    if sub_section:
+                        # This assertion makes MyPy happy. In this case, we
+                        # never set source_slice without also setting
+                        # subsection.
                         templater_logger.debug(
                             "        Attempting Subsplit [pre]: %s, %r",
                             sub_section,
@@ -780,7 +902,7 @@ class PythonTemplater(RawTemplater):
                                 IntermediateFileSlice(
                                     "compound",
                                     # Slicing is easy here, we have no choice
-                                    source_slice,
+                                    slice(starts[0], raw_idx),
                                     slice(starts[1], template_idx),
                                     sub_section,
                                 )
@@ -882,9 +1004,7 @@ class PythonTemplater(RawTemplater):
 
             if starts[1] < stops[1] and last_owu_idx is not None:
                 # Yield the end bit
-                templater_logger.debug(
-                    "        Attempting Subsplit [post]: %s", sub_section
-                )
+                templater_logger.debug("        Attempting Subsplit [post].")
                 yield from cls._split_uniques_coalesce_rest(
                     [
                         IntermediateFileSlice(
