@@ -12,7 +12,7 @@ class SQLFluffViolationReporter(BaseViolationReporter):
 
     def __init__(self):
         """Calls the base class constructor to set the object's name."""
-        super(SQLFluffViolationReporter, self).__init__("sqlfluff")
+        super().__init__("sqlfluff")
 
     @staticmethod
     def violations(src_path):
@@ -30,13 +30,17 @@ class SQLFluffViolationReporter(BaseViolationReporter):
         for violation in linted_path.get_violations():
             try:
                 # Normal SQLFluff warnings
-                message = violation.description
+                message = f"{violation.rule_code()}: {violation.description}"
             except AttributeError:
                 # Parse errors
                 message = str(violation)
-            result.append(Violation(violation.line_no(), message))
+            result.append(Violation(violation.line_no, message))
         return result
 
+    def measured_lines(self, src_path):
+        """Return list of the lines in src_path that were measured."""
+        return None
+
 
 @diff_cover_hookimpl
 def diff_cover_report_quality():
